@@ -34,8 +34,9 @@ pub enum Card {
     ScalarNil,
     CreateTable,
     Abort,
-    SetProperty(VarNode),
-    GetProperty(VarNode),
+    Len,
+    SetProperty,
+    GetProperty,
     ScalarInt(IntegerNode),
     ScalarFloat(FloatNode),
     StringLiteral(StringNode),
@@ -49,11 +50,14 @@ pub enum Card {
     ReadVar(VarNode),
     Repeat(LaneNode),
     While(LaneNode),
+    ForEach { variable: VarNode, lane: LaneNode },
 }
 
 impl Card {
     pub fn name(&self) -> &'static str {
         match self {
+            // Card::GetByKey => "GetByKey",
+            // Card::SetByKey => "SetByKey",
             Card::SetVar(_) => "SetLocalVar",
             Card::Pass => "Pass",
             Card::Add => "Add",
@@ -72,6 +76,7 @@ impl Card {
             Card::Or => "Either",
             Card::Xor => "Exclusive Or",
             Card::Abort => "Abort",
+            Card::Len => "Len",
             Card::ScalarInt(_) => "ScalarInt",
             Card::ScalarFloat(_) => "ScalarFloat",
             Card::StringLiteral(_) => "StringLiteral",
@@ -87,23 +92,25 @@ impl Card {
             Card::Repeat(_) => "Repeat",
             Card::While(_) => "While",
             Card::IfElse { .. } => "IfElse",
-            Card::GetProperty(_) => "GetProperty",
-            Card::SetProperty(_) => "SetProperty",
+            Card::GetProperty => "GetProperty",
+            Card::SetProperty => "SetProperty",
+            Card::ForEach { .. } => "ForEach",
         }
     }
 
     /// Translate this Card into an Instruction, if possible.
     /// Some cards expand to multiple instructions, these are handled separately
-    pub fn instruction(&self) -> Option<Instruction> {
+    pub(crate) fn instruction(&self) -> Option<Instruction> {
         match self {
             Card::IfElse { .. }
             | Card::ReadVar(_)
             | Card::SetVar(_)
             | Card::While(_)
-            | Card::Repeat(_) => None,
+            | Card::Repeat(_)
+            | Card::ForEach { .. } => None,
 
-            Card::GetProperty(_) => Some(Instruction::GetProperty),
-            Card::SetProperty(_) => Some(Instruction::SetProperty),
+            Card::GetProperty => Some(Instruction::GetProperty),
+            Card::SetProperty => Some(Instruction::SetProperty),
             Card::CreateTable => Some(Instruction::InitTable),
             Card::And => Some(Instruction::And),
             Card::Abort => Some(Instruction::Exit),
@@ -132,6 +139,7 @@ impl Card {
             Card::ClearStack => Some(Instruction::ClearStack),
             Card::ScalarNil => Some(Instruction::ScalarNil),
             Card::Return => Some(Instruction::Return),
+            Card::Len => Some(Instruction::Len),
         }
     }
 
@@ -141,6 +149,7 @@ impl Card {
     fn __instruction_to_node(instr: Instruction) {
         match instr {
             Instruction::SetGlobalVar
+            | Instruction::Len
             | Instruction::ReadGlobalVar
             | Instruction::GetProperty
             | Instruction::SetProperty
@@ -174,6 +183,10 @@ impl Card {
             | Instruction::Goto
             | Instruction::GotoIfTrue
             | Instruction::GotoIfFalse
+            | Instruction::Repeat
+            | Instruction::BeginRepeat
+            | Instruction::ForEach
+            | Instruction::BeginForEach
             | Instruction::Pass => {}
         };
     }
